#include "e.h"
#include "e_mod_main.h"

struct _E_Config_Dialog_Data
{
  int     resize;
  double  poll_time;   
  char   *line1_format;
  char   *line2_format;
  char   *line3_format;
  char   *hostname;
  char   *passwd;
  int     port;
};

/* Protos */
static void         *_create_data             (E_Config_Dialog *cfd);
static void          _free_data               (E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata);
static Evas_Object  *_basic_create_widgets    (E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cfdata);
static int           _basic_apply_data        (E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata);
static Evas_Object  *_advanced_create_widgets (E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cfdata);
static int           _advanced_apply_data     (E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata);

void
_config_isound_module(void) 
{
  E_Config_Dialog *cfd;
  E_Config_Dialog_View *v;
  char buf[4096];

  v = E_NEW(E_Config_Dialog_View, 1);

  v->create_cfdata = _create_data;
  v->free_cfdata = _free_data;
  v->basic.apply_cfdata = _basic_apply_data;
  v->basic.create_widgets = _basic_create_widgets;
  v->advanced.apply_cfdata = _advanced_apply_data;
  v->advanced.create_widgets = _advanced_create_widgets;

  snprintf(buf, sizeof(buf), "%s/module.edj", e_module_dir_get(isound_config->module));
  cfd = e_config_dialog_new(e_container_current_get(e_manager_current_get()),
                            _("Isound Configuration"), 
                            "E", "_e_mod_isound_config_dialog",
                            buf, 0, v, NULL);
  isound_config->config_dialog = cfd;
}

static void
_fill_data(E_Config_Dialog_Data *cfdata) 
{
  if (!isound_config) return;
  cfdata->resize    = isound_config->resize;
  cfdata->poll_time = isound_config->poll_time;
  cfdata->port      = isound_config->port;

  if (isound_config->line1_format) 
    cfdata->line1_format = strdup(isound_config->line1_format);
  if (isound_config->line2_format) 
    cfdata->line2_format = strdup(isound_config->line2_format);
  if (isound_config->line3_format) 
    cfdata->line3_format = strdup(isound_config->line3_format);
  if (isound_config->hostname) 
    cfdata->hostname = strdup(isound_config->hostname);
  if (isound_config->passwd) 
    cfdata->passwd = strdup(isound_config->passwd);
}

static void *
_create_data(E_Config_Dialog *cfd) 
{
  E_Config_Dialog_Data *cfdata;

  cfdata = E_NEW(E_Config_Dialog_Data, 1);
  cfdata->line1_format = NULL;
  cfdata->line2_format = NULL;
  cfdata->line3_format = NULL;
  cfdata->hostname     = NULL;
  cfdata->passwd       = NULL;

  _fill_data(cfdata);
  
  return cfdata;
}

static void
_free_data(E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata) 
{
  if (!isound_config) return;
  isound_config->config_dialog = NULL;
  free(cfdata);
}

static Evas_Object *
_basic_create_widgets(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cfdata) 
{
  Evas_Object *o, *of, *ob;

  o = e_widget_list_add(evas, 0, 0);
  of = e_widget_frametable_add(evas, _("Mpd Settings"), 0);

  ob = e_widget_label_add(evas, _("Hostname:"));
  e_widget_frametable_object_append(of, ob, 0, 0, 1, 1, 1, 0, 1, 0);
  ob = e_widget_entry_add(evas, &cfdata->hostname);
  e_widget_frametable_object_append(of, ob, 0, 1, 1, 1, 1, 0, 1, 0);

  ob = e_widget_label_add(evas, _("Port:"));
  e_widget_frametable_object_append(of, ob, 0, 2, 1, 1, 1, 0, 1, 0);
  //TODO use an entry
  ob = e_widget_slider_add(evas, 1, 0, _("%.0f"), 1024, 8192, 1, 0, NULL, &(cfdata->port), 200);
  e_widget_frametable_object_append(of, ob, 0, 3, 1, 1, 1, 0, 1, 0);

  ob = e_widget_label_add(evas, _("Password:"));
  e_widget_frametable_object_append(of, ob, 0, 4, 1, 1, 1, 0, 1, 0);
  ob = e_widget_entry_add(evas, &cfdata->passwd);
  e_widget_frametable_object_append(of, ob, 0, 5, 1, 1, 1, 0, 1, 0);

  e_widget_list_object_append(o, of, 1, 1, 0.5);
  return o;
}

static int 
_basic_apply_data(E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata) 
{
  if (!isound_config) return 0;
  if (isound_config->hostname) evas_stringshare_del(isound_config->hostname);
  if (isound_config->passwd)   evas_stringshare_del(isound_config->passwd);
  if (cfdata->hostname) isound_config->hostname = evas_stringshare_add(cfdata->hostname);
  if (cfdata->passwd)   isound_config->passwd   = evas_stringshare_add(cfdata->passwd);
  isound_config->port = cfdata->port;
  _isound_config_updated();
  e_config_save_queue();
  return 1;
}

static Evas_Object *
_advanced_create_widgets(E_Config_Dialog *cfd, Evas *evas, E_Config_Dialog_Data *cfdata) 
{
  Evas_Object *o, *of, *ob;

  /* Use Sliders for both cfg options */
  o = e_widget_list_add(evas, 0, 0);

  of = e_widget_frametable_add(evas, _("Mpd Settings"), 0);

  ob = e_widget_label_add(evas, _("Hostname:"));
  e_widget_frametable_object_append(of, ob, 0, 0, 1, 1, 1, 0, 1, 0);
  ob = e_widget_entry_add(evas, &cfdata->hostname);
  e_widget_frametable_object_append(of, ob, 0, 1, 1, 1, 1, 0, 1, 0);

  ob = e_widget_label_add(evas, _("Port:"));
  e_widget_frametable_object_append(of, ob, 0, 2, 1, 1, 1, 0, 1, 0);
  //TODO use an entry
  ob = e_widget_slider_add(evas, 1, 0, _("%.0f"), 1024, 8192, 1, 0, NULL, &(cfdata->port), 200);
  e_widget_frametable_object_append(of, ob, 0, 3, 1, 1, 1, 0, 1, 0);

  ob = e_widget_label_add(evas, _("Password:"));
  e_widget_frametable_object_append(of, ob, 0, 4, 1, 1, 1, 0, 1, 0);
  ob = e_widget_entry_add(evas, &cfdata->passwd);
  e_widget_frametable_object_append(of, ob, 0, 5, 1, 1, 1, 0, 1, 0);

  e_widget_list_object_append(o, of, 1, 1, 0.5);


  of = e_widget_frametable_add(evas, _("Advanced Settings"), 1);

  ob = e_widget_label_add(evas, _("Check song every:"));
  e_widget_frametable_object_append(of, ob, 0, 0, 1, 1, 1, 0, 1, 0);

  ob = e_widget_slider_add(evas, 1, 0, _("%1.1f seconds"), 0.2, 5.0, 0.1, 0, &(cfdata->poll_time), NULL, 200);
  e_widget_frametable_object_append(of, ob, 0, 1, 2, 1, 1, 0, 1, 0);

  ob = e_widget_check_add(evas, ("Resize at song change"), &(cfdata->resize));
  e_widget_frametable_object_append(of, ob, 0, 2, 1, 1, 1, 0, 1, 0);

  e_widget_list_object_append(o, of, 1, 1, 0.5);


  of = e_widget_frametable_add(evas, _("Display Format"), 0);

  ob = e_widget_label_add(evas, _("Write the wanted format to display songs informations."));
  e_widget_frametable_object_append(of, ob, 0, 0, 2, 1, 1, 0, 0, 0);
  ob = e_widget_label_add(evas, _("You can use any markup supported by the evas textblock."));
  e_widget_frametable_object_append(of, ob, 0, 1, 2, 1, 1, 0, 0, 0);
  ob = e_widget_label_add(evas, _("%a : artist"));
  e_widget_frametable_object_append(of, ob, 0, 2, 1, 1, 1, 0, 0, 0);
  ob = e_widget_label_add(evas, _("%b : album"));
  e_widget_frametable_object_append(of, ob, 0, 3, 1, 1, 1, 0, 0, 0);
  ob = e_widget_label_add(evas, _("%t : title"));
  e_widget_frametable_object_append(of, ob, 0, 4, 1, 1, 1, 0, 0, 0);
  ob = e_widget_label_add(evas, _("%m : time"));
  e_widget_frametable_object_append(of, ob, 0, 5, 1, 1, 1, 0, 0, 0);

  ob = e_widget_entry_add(evas, &cfdata->line1_format);
  e_widget_frametable_object_append(of, ob, 0, 6, 2, 1, 1, 0, 1, 0);

  ob = e_widget_entry_add(evas, &cfdata->line2_format);
  e_widget_frametable_object_append(of, ob, 0, 7, 2, 1, 1, 0, 1, 0);

  ob = e_widget_entry_add(evas, &cfdata->line3_format);
  e_widget_frametable_object_append(of, ob, 0, 8, 2, 1, 1, 0, 1, 0);

  e_widget_frametable_content_align_set(of, 0.0, 0.0);
  e_widget_list_object_append(o, of, 1, 1, 0.5);

  return o;
}

static int 
_advanced_apply_data(E_Config_Dialog *cfd, E_Config_Dialog_Data *cfdata) 
{
  if (!isound_config) return 0;
  if (isound_config->line1_format) evas_stringshare_del(isound_config->line1_format);
  if (isound_config->line2_format) evas_stringshare_del(isound_config->line2_format);
  if (isound_config->line3_format) evas_stringshare_del(isound_config->line3_format);
  if (cfdata->line1_format) isound_config->line1_format = evas_stringshare_add(cfdata->line1_format);
  if (cfdata->line2_format) isound_config->line2_format = evas_stringshare_add(cfdata->line2_format);
  if (cfdata->line3_format) isound_config->line3_format = evas_stringshare_add(cfdata->line3_format);
  if (isound_config->hostname) evas_stringshare_del(isound_config->hostname);
  if (isound_config->passwd)   evas_stringshare_del(isound_config->passwd);
  if (cfdata->hostname) isound_config->hostname = evas_stringshare_add(cfdata->hostname);
  if (cfdata->passwd)   isound_config->passwd   = evas_stringshare_add(cfdata->passwd);

  isound_config->port = cfdata->port;
  isound_config->resize = cfdata->resize;
  isound_config->poll_time = cfdata->poll_time;
  _isound_config_updated();
  e_config_save_queue();
  return 1;
}

